package com.xhkjedu.sexam.service.report;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.sexam.mapper.paperstudent.EPaperStudentMapper;
import com.xhkjedu.sexam.mapper.report.EReportSchoolMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassMapper;
import com.xhkjedu.sexam.mapper.reportclass.ERclassRankMapper;
import com.xhkjedu.sexam.mapper.reportstu.ERstudentMapper;
import com.xhkjedu.sexam.model.reportstu.ERstudent;
import com.xhkjedu.sexam.vo.report.ERPaperVo;
import com.xhkjedu.sexam.vo.report.ERankParamVo;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author ywx
 * @className EReportClassService
 * @description 班级报告
 * @date 2022/8/8 9:46
 **/
@Service
public class EReportClassService {
    //@Resource
    //private ERbaseMapper eRbaseMapper;
    //@Resource
    //private ERsubjectMapper eRsubjectMapper;
    @Resource
    private ERstudentMapper eRstudentMapper;
    @Resource
    private ERclassMapper eRclassMapper;
    @Resource
    private ERclassRankMapper eRclassRankMapper;
    @Resource
    private EReportGenerateService eReportGenerateService;
    @Resource
    private EReportSchoolMapper eReportSchoolMapper;
    @Resource
    private EPaperStudentMapper ePaperStudentMapper;

    //考试概况-学科成绩
    public List<Map> listXkCj(Integer examid, Integer classid) {
        return eRclassMapper.listByExamAndClassId(examid, classid);
    }

    //成绩单
    public List<Map> listCjd(ERstudent stu) {
        return eRstudentMapper.listByExamId(stu);
    }

    //成绩分析-搜索
    public List<Map> listCjFxForSearch(ERankParamVo rank) {
        return eRclassRankMapper.listByExamAndClassId(rank);
    }

    //成绩分析-设置
    @Transactional(rollbackFor = Exception.class)
    public List<Map> listCjFxForSet(ERankParamVo rank) {
        Integer examid = rank.getExamid();
        String subjectid = rank.getSubjectid();
        Integer createid = rank.getCreateid();
        Integer ranktype = rank.getRanktype();
        eReportSchoolMapper.delByExamAndSubjectId(examid, subjectid, ranktype);//删除考试等级报告
        List<ERPaperVo> papers = eReportSchoolMapper.listByExamAndSubjectId(examid, subjectid);//考试试卷
        List<ERstudent> students = ePaperStudentMapper.listByExamAndSubjectId(examid, subjectid);//考试试卷学生
        eReportGenerateService.setClassGradeRank(examid, papers, students, rank.getRranks(), createid);//生成考试等级报告
        return eRclassRankMapper.listByExamId(rank);//考试等级报告列表
    }

    //单科概况
    public Map getDkGk(ERankParamVo rank) {
        Map map = eRclassMapper.findById(rank);//班级概况
        List<Map> pjfList = eRclassMapper.listPjf(rank);//平均分
        map.put("pjfList", pjfList);
        List<Map> cjdjList = eRclassMapper.listCjDj(rank);//成绩等级分布
        map.put("cjdjList", cjdjList);
        rank.setRanktype(1);//大幅进步
        List<Map> gzxsList = eRclassMapper.listGzXs(rank);//需关注学生
        map.put("gzxsList", gzxsList);
        List<Map> gpctList = eRclassMapper.listGpCt(rank);//高频错题
        for (Map ct : gpctList) {
            //高频错误项
            Integer ctype = (Integer) ct.get("ctype");
            String gpcwx = "";
            if (N_Utils.isObjectiveQuestion(ctype)) {
                String answerjson = ct.get("answerjson").toString();//学生作答情况
                String answer = JSON.parseArray(ct.get("answer").toString(), String.class).get(0);//正确答案
                Map amap = JSON.parseObject(answerjson, Map.class);
                List<Map<String, Integer>> list = new ArrayList<>();
                Integer cwnum = 0;//作答错误学生数量
                for (Object o : amap.keySet()) {
                    List<Map> o1 = (List<Map>) amap.get(o);
                    int answernum = o1.size();//作答学生数量
                    String xx = o.toString();//选项
                    if (!answer.equals(xx) && cwnum.compareTo(answernum) == -1) {
                        cwnum = answernum;
                        gpcwx = xx;
                    }
                }
            }
            ct.put("gpcwx", gpcwx);

            //知识点
            Object pointjson = ct.get("pointjson");

            ct.remove("answerjson");
            ct.remove("ctype");
            ct.remove("answer");
            ct.remove("pointjson");

            List<String> pointnames = new ArrayList<>();
            if (N_Utils.isNotEmpty(pointjson)) {
                List<Map> list = JSON.parseArray(pointjson.toString(), Map.class);
                Integer eptqid = (Integer) ct.get("eptqid");
                for (Map p : list) {
                    String ques = p.get("ques").toString();
                    String pointname = p.get("pointname").toString();
                    List<Map> qlist = JSON.parseArray(ques, Map.class);
                    for (Map q : qlist) {
                        List<Integer> eptqids = JSON.parseArray(q.get("eptqids").toString(), Integer.class);
                        if (N_Utils.numInList(eptqid, eptqids)) {
                            pointnames.add(pointname);
                            break;
                        }
                    }
                }
            }
            ct.put("pointnames", pointnames);
        }
        map.put("gpctList", gpctList);
        return map;
    }

    //单科概况-需关注学生
    public List<Map> listGzXs(ERankParamVo rank) {
        return eRclassMapper.listGzXs(rank);
    }
}
